<?php

namespace Ig\IgCookies\Controller;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use Psr\Http\Message\ResponseInterface;
use TYPO3\CMS\Core\Localization\LanguageServiceFactory;
use TYPO3\CMS\Core\Http\JsonResponse;

class MainController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    protected $languageFile = 'LLL:EXT:ig_cookies/Resources/Private/Language/locallang.xlf';
    protected $labelKeys = ['title', 'description'];

    /**
     * action cookie: display info to accept cookies
     */
    public function consentAction(): ResponseInterface
    {
        $site = $GLOBALS['TYPO3_REQUEST']->getAttribute('site');
        $rootPageUid = $site->getRootPageId();
         $this->uriBuilder
             ->reset()
             ->setTargetPageUid($rootPageUid)
             ->setTargetPageType(1690140404);
         // v12
         if (method_exists($this, 'getRequest')) {
             $this->uriBuilder->setRequest($this->getRequest());
         }
         $configUrl = $this->uriBuilder->build();
        
        $this->view->assign('configUrl', $configUrl);
        return $this->htmlResponse($this->view->render());
    }


    /**
     * config action
     */
    public function configAction(): ResponseInterface
    {
        $this->initServices();
        $this->translations = [
            'alwaysRequired' => $this->translate('LLL:EXT:ig_cookies/Resources/Private/Language/locallang.xlf:always-required'),
        ];
        $response = [
            'purposes' => $this->purposesArray,
            'services' => $this->servicesArray,
            'translations' => $this->translations,
        ];
        return new JsonResponse($response);
    }

    
    public function getTranslatedLabels($purpose, &$arr)
    {
        foreach ($this->labelKeys as $label) {
            if ($purpose[$label]) {
                $ll = trim($purpose[$label]);
                if (substr($ll, 0, 8) == 'LLL:EXT:') {
                    $arr[$label] = $this->translate($ll);
                } else if (strpos($ll, '.') && !strpos($ll, ' ')) {
                    $arr[$label] = $this->translate($this->languageFile . ':' . $ll);
                } else {
                    $arr[$label] = $ll;
                }
            }
        }
    }
    public function translateArray(&$arr)
    {
        foreach ($arr as $name => &$value) {
            if (is_array($value)) {
                $this->translateArray($value);
            } elseif ($value) {
                $ll = trim($value);
                if (substr($ll, 0, 8) == 'LLL:EXT:') {
                    $value = $this->translate($ll);
                }
            }
        }
    }
    private static function translate(string $lll): string
    {
        $languageServiceFactory = GeneralUtility::makeInstance(
            LanguageServiceFactory::class
        );
        $request = $GLOBALS['TYPO3_REQUEST'];
        $languageService = $languageServiceFactory->createFromSiteLanguage(
            $request->getAttribute('language')
            ?? $request->getAttribute('site')->getDefaultLanguage()
        );
        return $languageService->sL($lll);
    }
    protected function initServices(): void
    {
        $this->purposesArray = [];
        $this->servicesArray = [];
        $purposes = array_filter($this->settings['purposes']);  //all purposes
        $activePurposes = [];
        $usedPurposes = [];
        $this->services = array_filter($this->settings['services']);
        foreach ($this->services as $sorting => &$service) {
            $service['contextualConsentOnly'] = filter_var($service['contextualConsentOnly'] ?? '', FILTER_VALIDATE_BOOLEAN);
            $this->translateArray($service);
            if (!isset($service['policyText'])) {
                $service['policyText'] = $this->translate('LLL:EXT:ig_cookies/Resources/Private/Language/locallang.xlf:serviceContextualButton.moreText');
            }
            $this->servicesArray[] = $service;
            $purposeName = $service['purpose'];
            if (!isset($activePurposes[$purposeName]) && isset($purposes[$purposeName])) {
                $activePurposes[$purposeName] = true;
            }
        }
        // keep order
        foreach ($purposes as $name => $purpose) {
            if(isset($activePurposes[$name])) {
                $usedPurposes[$name] = $purpose;
            }
        }
        if ($purposes['languageFile'] ?? false) {
            $this->languageFile = $purposes['languageFile'];
            unset($purposes['languageFile']);
        }
        foreach ($usedPurposes as $name => $purpose) {
            $arr = $purpose;
            //$arr['name'] = trim($purpose['name']);
            $arr['name'] = $name;
            $arr['required'] = filter_var($purpose['required'] ?? '', FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE);
            $this->translateArray($arr);
            /*
            $arr['services'] = [];
            foreach ($this->servicesArray as $serviceEntry) {
                if ($name == trim($serviceEntry['purpose'])) {
                    $arr['services'][] = $serviceEntry;
                }
            }
            */
            $this->purposesArray[] = $arr;
        }
    }



}
